@@ -234,6 +234,13 @@ fn write_output<R: io::Read, W: io::Write>(
     Ok(())
 }
 
+#[inline]
+fn valid_plugin_name(plugin_name: &str) -> bool {
+    plugin_name
+        .bytes()
+        .all(|b| b.is_ascii_alphanumeric() | matches!(b, b'+' | b'-' | b'.' | b'_'))
+}
+
 fn decrypt(opts: AgeOptions) -> Result<(), error::DecryptError> {
     if opts.armor {
         return Err(error::DecryptError::ArmorFlag);
@@ -270,6 +277,12 @@ fn decrypt(opts: AgeOptions) -> Result<(), error::DecryptError> {
     let identities = if plugin_name.is_empty() {
         read_identities(opts.identity, opts.max_work_factor, &mut stdin_guard)?
     } else {
+        if !valid_plugin_name(plugin_name) {
+            return Err(age::DecryptError::MissingPlugin {
+                binary_name: plugin_name.into(),
+            }
+            .into());
+        }
         // Construct the default plugin.
         vec![Box::new(plugin::IdentityPluginV1::new(
             plugin_name,
